{% load i18n %}
<div class="sequence proctored-exam completed" data-exam-id="{{exam_id}}">
  <h3>
    {% blocktrans %}
      You have submitted this proctored exam for review
    {% endblocktrans %}
  </h3>
  <ul>
    <li> {% trans "Your recorded data should now be uploaded for review." %}</li>
      <ul>
        <li> {% trans "If the proctoring software window is still open, close it now and confirm that you want to quit the application." %}</li>
      </ul>
      <li> {% trans "Proctoring results are usually available within 5 business days after you submit your exam." %}</li>
  </ul>

  {% include 'proctored_exam/visit_exam_content.html' %}

  <p>
    {% blocktrans %}
      If you have questions about the status of your proctored exam results, contact {{ platform_name }} Support.
    {% endblocktrans %}
  </p>
</div>
{% include 'proctored_exam/footer.html' %}
