{% load i18n %}
<div class="failure sequence proctored-exam entrance" data-exam-id="{{exam_id}}">
  <h3>
    {% blocktrans %}
      This exam is proctored
    {% endblocktrans %}
  </h3>
  <p>
    {% blocktrans %}
    You did not satisfy the requirements for taking this exam with proctoring.
    {% endblocktrans %}
  </p>
  <p>
    {% blocktrans %}
    You did not satisfy the following prerequisites:
    {% endblocktrans %}
    <ol style='list-style-type: disc'>
    {% for requirement in prerequisite_status.failed_prerequisites %}
      <li>
        {% if requirement.jumpto_url %}
          <a href='{{requirement.jumpto_url}}'>{{requirement.display_name}}</a>
        {% else %}
          {{requirement.display_name}}
        {% endif %}
      </li>
    {% endfor %}
    </ol>
  </p>

  {% if allow_proctoring_opt_out %}
    <p>
      {% blocktrans %}
        Due to unsatisfied prerequisites, you can only take this exam without proctoring.
      {% endblocktrans %}
    </p>

    {% include 'proctored_exam/proctoring_opt_out_button.html' %}
  {% endif %}
  <p>
    {% blocktrans %}
      If you have questions about the status of your requirements, contact {{ platform_name }} Support.
    {% endblocktrans %}
  </p>
</div>
{% include 'proctored_exam/confirm-decline.html' %}
{% include 'proctored_exam/footer.html' %}
